@@ -56,69 +56,8 @@ class CFToolkit(object):
         self.__width = '100%'
         self.__height = '400px'
         self.__iframe_height = '300px'
-
         display(Javascript(StaticFilesManager.js('jupyter-notebook-active-cell-change')))
 
-        # region Google Colab support
-        try:
-            from google.colab import output
-
-            def get_data_sources():
-                try:
-                    with output.use_tags('get_data_sources'):
-                        get_ipython().ex("import pandas as pd")
-                        get_ipython().ex("import json")
-                        data_sources = get_ipython().ev("json.dumps([{'id': g, 'name': g, 'providerType': 'pandas-dataframe', 'type': 'DATASET' } for g in globals() if isinstance(eval(g), pd.core.frame.DataFrame)])")
-                        return data_sources
-                    output.clear(output_tags='get_data_sources')
-                except Exception as e:
-                    raise Exception(e)
-
-            def get_data_source(df_name):
-                try:
-                    with output.use_tags('get_data_source'):
-                        df = self.__get_dataframe_by_name(df_name)
-                        return Provider(df).get_data_source(df_name)
-                    output.clear(output_tags='get_data_source')
-                except Exception as e:
-                    raise Exception(e)
-
-            def run_count_query(df_name, json_config):
-                try:
-                    with output.use_tags('run_count_query'):
-                        df = self.__get_dataframe_by_name(df_name)
-                        return Provider(df, json.loads(json_config)).run_count_query()
-                    output.clear(output_tags='run_count_query')
-                except Exception as e:
-                    raise Exception(e)
-
-            def run_raw_query(df_name, json_config):
-                try:
-                    with output.use_tags('run_raw_query'):
-                        df = self.__get_dataframe_by_name(df_name)
-                        return Provider(df, json.loads(json_config)).run_raw_query()
-                    output.clear(output_tags='run_raw_query')
-                except Exception as e:
-                    raise Exception(e)
-
-            def visualize(df_name, json_config):
-                try:
-                    with output.use_tags('visualize'):
-                        df = self.__get_dataframe_by_name(df_name)
-                        return Provider(df, json.loads(json_config)).visualize()
-                    output.clear(output_tags='visualize')
-                except Exception as e:
-                    raise Exception(e)
-
-            output.register_callback('get_data_sources', get_data_sources)
-            output.register_callback('get_data_source', get_data_source)
-            output.register_callback('run_count_query', run_count_query)
-            output.register_callback('run_raw_query', run_raw_query)
-            output.register_callback('visualize', visualize)
-        except ImportError:
-            pass
-        # endregion
-
     @property
     def get_source(self):
         return self.__source
@@ -145,6 +84,7 @@ class CFToolkit(object):
 
             display(Javascript(StaticFilesManager.js('execute-python')))
             display(Javascript(StaticFilesManager.js('send-message')))
+            display(Javascript(StaticFilesManager.js('kaggle-specs')))
             clear_output()
         except Exception as e:
             print(f'{Style.BRIGHT}{Fore.RED}{e}{Style.RESET_ALL}')
@@ -168,6 +108,68 @@ class CFToolkit(object):
 
     # endregion
 
+    def __init_colab(self):
+        # region Google Colab support
+        try:
+            from google.colab import output
+
+            def get_data_sources():
+                try:
+                    output.clear(output_tags='get_data_sources')
+                    with output.use_tags('get_data_sources'):
+                        get_ipython().ex("import pandas as pd")
+                        get_ipython().ex("import json")
+                        data_sources = get_ipython().ev("json.dumps([{'id': g, 'name': g, 'providerType': 'pandas-dataframe', 'type': 'DATASET' } for g in globals() if isinstance(eval(g), pd.core.frame.DataFrame)])")
+                        return data_sources
+                except Exception as e:
+                    raise Exception(e)
+
+            def get_data_source(df_name):
+                try:
+                    output.clear(output_tags='get_data_source')
+                    with output.use_tags('get_data_source'):
+                        df = self.__get_dataframe_by_name(df_name)
+                        return Provider(df).get_data_source(df_name)
+                except Exception as e:
+                    raise Exception(e)
+
+            def run_count_query(df_name, json_config):
+                try:
+                    output.clear(output_tags='run_count_query')
+                    with output.use_tags('run_count_query'):
+                        df = self.__get_dataframe_by_name(df_name)
+                        return Provider(df, json.loads(json_config)).run_count_query()
+                except Exception as e:
+                    raise Exception(e)
+
+            def run_raw_query(df_name, json_config):
+                try:
+                    output.clear(output_tags='run_raw_query')
+                    with output.use_tags('run_raw_query'):
+                        df = self.__get_dataframe_by_name(df_name)
+                        return Provider(df, json.loads(json_config)).run_raw_query()
+                except Exception as e:
+                    raise Exception(e)
+
+            def visualize(df_name, json_config):
+                try:
+                    output.clear(output_tags='visualize')
+                    with output.use_tags('visualize'):
+                        df = self.__get_dataframe_by_name(df_name)
+                        return Provider(df, json.loads(json_config)).visualize()
+                except Exception as e:
+                    raise Exception(e)
+
+            output.register_callback('get_data_sources', get_data_sources)
+            output.register_callback('get_data_source', get_data_source)
+            output.register_callback('run_count_query', run_count_query)
+            output.register_callback('run_raw_query', run_raw_query)
+            output.register_callback('visualize', visualize)
+        except ImportError:
+            pass
+        # endregion
+        
+
     def provider(self, dataframe, json_config=None):
         """
         Creates a new instance of the Provider class from a given dataframe and json config
@@ -178,27 +180,36 @@ class CFToolkit(object):
         self.__provider = Provider(dataframe, json_config)
         return self.__provider
 
-    def studio(self, app=None, url="https://chartfactor.com/studio/"):
+    def studio(self, app=None, url=None):
         """
         Opens an instance of ChartFactor Studio inside of an IFrame.
         :param app: The name of a Studio app. A new application will be created if app=None of not found.
         :param url: The ChartFactor Studio host url. By default will be "https://chartfactor.com/studio/".
         :return: IFrame
         """
+        if url is None:
+            url = f'https://chartfactor.com/cf/studio/{version}/'
+            
         self.__add_js_functions()
-        time.sleep(0.1)
-
+        time.sleep(0.2)
+        
+        self.__kernel_id = get_ipython().ev("re.search('kernel-(.*).json', ipykernel.connect.get_connection_file()).group(1)")
         self.__prompt_number = get_ipython().ev("len(In)-1")
+        iframe_id = f'iframe{self.__prompt_number}'
+        
         # Getting active cell metadata to check if is a new cell
         # or already contains a dashboard inside
-        active_cell_id = get_ipython().ev('cfpy_active_cell_id')
-        iframe_id = f'iframe{self.__prompt_number}'
-        self.__kernel_id = get_ipython().ev("re.search('kernel-(.*).json', ipykernel.connect.get_connection_file()).group(1)")
+        is_kernel_id = 'false'
+        try:
+            active_cell_id = get_ipython().ev('cfpy_active_cell_id')
+        except:
+            active_cell_id = self.__kernel_id
+            is_kernel_id = 'true'
 
         if isinstance(self.__kernel_id, str) and self.__kernel_id != '':
             iframe_id = f'{iframe_id}_{self.__kernel_id}'
 
-        query_string = f'appName={urllib.parse.quote("New Application") if app is None else urllib.parse.quote(app)}&cellId={active_cell_id}'
+        query_string = f'appName={urllib.parse.quote("New Application") if app is None else urllib.parse.quote(app)}&cellId={active_cell_id}&isKernelId={is_kernel_id}'
         time.sleep(0.3)
         display(HTML(
             f'<iframe id={iframe_id} name={iframe_id} src={url}dashboard.html#/{query_string} width=100% height=764 frameborder=0 allow="clipboard-write *;"></iframe>'))
@@ -374,6 +385,23 @@ class CFToolkit(object):
             raise Exception("The source() function must be called before the groupby() one")
         return self
 
+    def colgroupby(self, *args):
+        """
+        Sets the attributes to be perform the top-n column group query in the current chart.
+        :param args: The Attribute instances to be grouped in the column of the current chart.
+        :return: The current CFToolkit instance.
+        """
+        if self.__provider is not None:
+            list_args = list(args)
+            if len(list_args) > 0 and isinstance(list_args[0], list):
+                parameters = list_args[0]
+            else:
+                parameters = list(args)
+            self.__provider.colgroupby(parameters)
+        else:
+            raise Exception("The source() function must be called before the colgroupby() one")
+        return self
+
     def rows(self, *args):
         if self.__provider is not None:
             list_args = list(args)
