@@ -1,3 +1,4 @@
+import math
 import re
 import numpy as np
 import pandas as pd
@@ -137,6 +138,7 @@ class Provider(object):
         self.__client_filters = []
         self.__static_filters = []
         self.__groups = []
+        self.__col_groups = []
         self.__metrics = []
         self.__original_metrics = []
         self.__comparative_metrics = []
@@ -211,6 +213,10 @@ class Provider(object):
     def get_groups(self):
         return self.__groups
 
+    @property
+    def get_colgroups(self):
+        return self.__col_groups
+
     @property
     def get_metrics(self):
         return self.__metrics
@@ -363,6 +369,10 @@ class Provider(object):
                     groups = cmms.__deep_get__(json_data, 'config.groups') or []
                     self.__groups = self.__get_groups__(groups, json_data)
 
+                    # Process the colgroups
+                    col_groups = cmms.__deep_get__(json_data, 'config.colgroups') or []
+                    self.__col_groups = self.__get_groups__(col_groups, json_data)
+
                     # Process the fields
                     fields = cmms.__deep_get__(json_data, 'config.fields') or []
                     for fl in fields:
@@ -762,6 +772,8 @@ class Provider(object):
             return self.__location and self.__precision
         elif type == 'pivot':
             return self.__aggregate and len(self.__rows) > 0
+        elif type == 'colgroups':
+            return len(self.__groups) > 0 and len(self.__col_groups) > 0
         else:
             return False
 
@@ -979,6 +991,13 @@ class Provider(object):
                 group_df[g.get_name] = group_df[g.get_name].dt.tz_localize(None)  # Removing time offset
                 group_df[g.get_name] = group_df[g.get_name].dt.to_period(granularity_g).dt.to_timestamp()
 
+            original_type = self.get_original_type(g.get_name)
+            is_tuple = isinstance(group_df[g.get_name][0], tuple)
+            is_list = isinstance(group_df[g.get_name][0], list)
+
+            if original_type == 'mixed' and (is_tuple or is_list):
+                group_df = group_df.explode(g.get_name).reset_index(drop=True)
+
             group = group_df.groupby(g.get_name, dropna=False).agg(metrics).copy().reset_index()
             group.columns = [sanitize('_'.join(filter(None, x))) for x in group.columns]  # Renaming columns
 
@@ -1099,6 +1118,8 @@ class Provider(object):
             sort_by = f'{sanitize(sort_name_g1)}_{sort_func_g1}'
             if sort_name_g1 == g1.get_name:
                 sort_by = sanitize(sort_name_g1)
+                if sort_func_g1 == 'count':
+                    sort_by += '_count'
             sub_query2 = sub_query2.sort_values([sort_by], ascending=[asc_g1]).iloc[
                          :limit_g1].reset_index(drop=True)
 
@@ -1106,12 +1127,12 @@ class Provider(object):
             for m in self.__metrics:
                 metrics_select += f"sq1.{sanitize(str(m.name))}_{m.func},\n"
 
-            if sort_name_g1 == g1.get_name:
+            if sort_name_g1 == g1.get_name and sort_func_g1 != 'count':
                 first_sort = sanitize(str(sort_name_g1))
             else:
                 first_sort = f"{sanitize(str(sort_name_g1))}_{sort_func_g1}"
 
-            if sort_name_g2 == g2.get_name:
+            if sort_name_g2 == g2.get_name and sort_func_g2 != 'count':
                 second_sort = sanitize(str(sort_name_g2))
             else:
                 second_sort = f"{sanitize(str(sort_name_g2))}_{sort_func_g2}"
@@ -1160,6 +1181,173 @@ class Provider(object):
         except Exception as e:
             raise Exception(e)
 
+    def __process_colgroups__(self):
+        row_group = self.__groups[0]
+        limit_row_group = row_group.get_limit
+        sort_dir_row_group = row_group.get_sort.get('dir')
+        asc_row_group = True if sort_dir_row_group == cnts.ASC else False
+        sort_name_row_group = row_group.get_sort.get('name')
+        sort_func_row_group = row_group.get_sort.get('func')
+        granularity_row_group = cnts.GRANULARITY_TRANSLATED.get(row_group.get_granularity)
+        tz_row_group = row_group.get_tz
+
+        col_group = self.__col_groups[0]
+        limit_col_group = col_group.get_limit
+        sort_dir_col_group = col_group.get_sort.get('dir')
+        asc_col_group = True if sort_dir_col_group == cnts.ASC else False
+        sort_name_col_group = col_group.get_sort.get('name')
+        sort_func_col_group = col_group.get_sort.get('func')
+        granularity_col_group = cnts.GRANULARITY_TRANSLATED.get(col_group.get_granularity)
+        tz_col_group = col_group.get_tz
+
+        columns = [row_group.get_name, col_group.get_name]
+
+        metrics = {}
+        metrics_sql_agg_sub2 = []
+        for m in self.__metrics:
+            columns.append(m.name)
+
+            # Adding all sql codes for metrics e.g: "sum(commission) as commission_sum"
+            if m.func != 'unique':
+                metric_sql_agg = f'{m.func}({sanitize(str(m.name))}) AS {sanitize(str(m.name))}_{m.func}'
+                if metric_sql_agg not in metrics_sql_agg_sub2:
+                    metrics_sql_agg_sub2.append(metric_sql_agg)
+
+            if m.name in metrics:
+                metrics[m.name].append(m.func)
+            else:
+                metrics[m.name] = ['nunique'] if m.func == cnts.UNIQUE else ['size'] if m.func == cnts.COUNT else [
+                    m.func]
+
+        # Adding total count
+        if row_group.get_name in metrics:
+            if 'size' not in metrics[row_group.get_name]:
+                metrics[row_group.get_name].append('size')
+        else:
+            metrics[row_group.get_name] = ['size']
+
+        groups_df = self.__data.loc[:, self.__data.columns.isin(columns)].reset_index(drop=True)
+        groups_df['total_cf_count'] = 1  # Column to calculate the count in queries
+
+        # Converting row_group column to the specified granularity
+        if row_group.get_granularity:
+            convert_to_datetime_efficiently(groups_df, row_group.get_name)
+            if tz_row_group != 'UTC':
+                groups_df[row_group.get_name] = groups_df[row_group.get_name].dt.tz_convert(tz_row_group)
+            groups_df[row_group.get_name] = groups_df[row_group.get_name].dt.tz_localize(None)  # Removing time offset
+            groups_df[row_group.get_name] = groups_df[row_group.get_name].dt.to_period(granularity_row_group).dt.to_timestamp()
+
+        # Converting col_group column to the specified granularity
+        if col_group.get_granularity:
+            convert_to_datetime_efficiently(groups_df, col_group.get_name)
+            if tz_col_group != 'UTC':
+                groups_df[col_group.get_name] = groups_df[col_group.get_name].dt.tz_convert(tz_col_group)
+            groups_df[col_group.get_name] = groups_df[col_group.get_name].dt.tz_localize(None)  # Removing time offset
+            groups_df[col_group.get_name] = groups_df[col_group.get_name].dt.to_period(granularity_col_group).dt.to_timestamp()
+
+        row_group_query = groups_df.groupby([row_group.get_name], dropna=False).agg(metrics).copy().reset_index()
+        row_group_query.columns = [sanitize('_'.join(filter(None, x))) for x in row_group_query.columns]
+        row_group_sort_by = f'{sanitize(sort_name_row_group)}_{sort_func_row_group}'
+        if sort_name_row_group == row_group.get_name:
+            row_group_sort_by = sanitize(sort_name_row_group)
+            if sort_func_row_group == 'count':
+                row_group_sort_by += '_count'
+        row_group_query = row_group_query.sort_values([row_group_sort_by], ascending=[asc_row_group]).iloc[:limit_row_group].reset_index(drop=True).reset_index()
+
+        col_group_query = groups_df.groupby([col_group.get_name], dropna=False).agg(metrics).copy().reset_index()
+        col_group_query.columns = [sanitize('_'.join(filter(None, x))) for x in col_group_query.columns]
+        col_group_sort_by = f'{sanitize(sort_name_col_group)}_{sort_func_col_group}'
+        if sort_name_col_group == col_group.get_name:
+            col_group_sort_by = sanitize(sort_name_col_group)
+            if sort_func_col_group == 'count':
+                col_group_sort_by += '_count'
+        col_group_query = col_group_query.sort_values([col_group_sort_by], ascending=[asc_col_group]).iloc[:limit_col_group].reset_index(drop=True).reset_index()
+
+        groups_df.columns = [sanitize(x) for x in groups_df.columns]
+
+        final_metrics_select = []
+        cross_row_metrics_select = []
+        for m in self.__metrics:
+            if m.func != 'unique':
+                final_metric_select = f"_sub2.{sanitize(str(m.name))}_{m.func}"
+                cross_metric_select = f"_main.{sanitize(str(m.name))}_{m.func}"
+
+                if final_metric_select not in final_metrics_select:
+                    final_metrics_select.append(final_metric_select)
+                if cross_metric_select not in cross_row_metrics_select:
+                    cross_row_metrics_select.append(cross_metric_select)
+
+        final_query = f"""
+        SELECT _sub1.{row_group.get_name}, _sub1.{col_group.get_name}, {', '.join(final_metrics_select)}, count
+        FROM 
+        (
+            SELECT _crossRow.{row_group.get_name}, _crossCol.{col_group.get_name}, _crossRow.{row_group_sort_by}, rank
+            FROM (
+                SELECT _main.{row_group.get_name}, {', '.join(cross_row_metrics_select)}
+                    FROM row_group_query _main
+                ) _crossRow
+               CROSS JOIN (
+                    SELECT *, ROW_NUMBER() OVER(ORDER BY {col_group_sort_by} {sort_dir_col_group}) AS rank
+                    FROM col_group_query 
+                    ORDER BY rank ASC
+               ) _crossCol   
+            GROUP BY _crossRow.{row_group.get_name}, _crossCol.{col_group.get_name}, _crossRow.{row_group_sort_by}, rank
+        ) _sub1
+        LEFT JOIN
+        (			
+            SELECT _main.{row_group.get_name}, _main.{col_group.get_name}, _colGroupSub.rank, {', '.join(cross_row_metrics_select)}, count
+            FROM
+            (
+                SELECT *, ROW_NUMBER() OVER(ORDER BY {col_group_sort_by} {sort_dir_col_group}) AS rank 
+                FROM col_group_query
+            ) _colGroupSub
+            LEFT JOIN
+            (
+                SELECT _main.{row_group.get_name}, _main.{col_group.get_name}, {', '.join(metrics_sql_agg_sub2)}, COUNT(*) AS count
+                FROM groups_df as _main
+                GROUP BY _main.{row_group.get_name}, _main.{col_group.get_name}
+            ) _main
+            on _colGroupSub.{col_group.get_name} = _main.{col_group.get_name}
+            GROUP BY _main.{row_group.get_name}, _main.{col_group.get_name}, _colGroupSub.rank, {', '.join(cross_row_metrics_select)}, count
+        ) _sub2
+        ON coalesce(_sub1.{row_group.get_name}, '&&null&&') = coalesce(_sub2.{row_group.get_name}, '&&null&&') and 
+        coalesce(_sub1.{col_group.get_name}, '&&null&&') = coalesce(_sub2.{col_group.get_name}, '&&null&&') 
+        ORDER BY _sub1.{row_group_sort_by} {sort_dir_row_group}, _sub1.rank ASC
+        """
+
+        groups = sqldf(final_query, locals())
+
+        response = {
+            'data': [],
+            "visualization": self.__visualization
+        }
+
+        for row in groups.itertuples():
+            row = row._asdict()
+            current_metrics = {}
+            for m in self.__metrics:
+                if m.func != 'unique':
+                    if m.name not in current_metrics:
+                        current_metrics[m.name] = {}
+                    if math.isnan(row[f"{sanitize(str(m.name))}_{m.func}"]):
+                        current_metrics[m.name][m.func.replace('mean', 'avg')] = None
+                    else:
+                        current_metrics[m.name][m.func.replace('mean', 'avg')] = round(
+                            float(row[f"{sanitize(str(m.name))}_{m.func}"]), 2)
+
+                group1 = None if str(row[sanitize(row_group.get_name)]) in ['None', 'nan'] else str(row[sanitize(row_group.get_name)])
+                group2 = None if str(row[sanitize(col_group.get_name)]) in ['None', 'nan'] else str(row[sanitize(col_group.get_name)])
+
+            response.get('data').append({
+                'group': [group1, group2],
+                'current': {
+                    'count': None if math.isnan(row['count']) else int(row['count']),
+                    'metrics': current_metrics
+                }
+            })
+
+        return json.dumps({'data': response})
+
     def __process_boxplot__(self):
         try:
             if len(self.__groups) > 0:
@@ -1775,6 +1963,8 @@ class Provider(object):
                         return self.__process_pivot__()
                     elif self.__is_type__('raw_query'):
                         return json.dumps({'data': response})
+                    elif self.__is_type__('colgroups'):
+                        return self.__process_colgroups__()
                     elif len(self.__groups) == 0 and len(self.__metrics) > 0:
                         return self.__process_metrics_totals__()
                     elif len(self.__groups) == 1:
@@ -1807,8 +1997,10 @@ class Provider(object):
             columns = self.__data.columns
 
             for column in columns:
+                dtype = self.__data.dtypes[column]
+                sample = self.__data[column].iloc[:1000]
                 try:
-                    original_type = types.infer_dtype(self.__data[column])
+                    original_type = dtype.name if dtype.name != 'object' else types.infer_dtype(sample)
                 except TypeError as e:
                     original_type = 'unknown'
 
@@ -1817,16 +2009,16 @@ class Provider(object):
                 if original_type == 'date':
                     current_type = "TIME"
                 elif original_type in ['timedelta64', 'timedelta', 'time'] or \
-                        types.is_timedelta64_dtype(self.__data[column]) or \
-                        types.is_timedelta64_ns_dtype(self.__data[column]):
+                        types.is_timedelta64_dtype(dtype) or \
+                        types.is_timedelta64_ns_dtype(dtype):
                     current_type = "ATTRIBUTE"
-                elif original_type == 'datetime' or types.is_datetime64_any_dtype(self.__data[column]):
+                elif original_type == 'datetime' or types.is_datetime64_any_dtype(dtype):
                     current_type = "TIME"
                     original_type = 'datetime'
-                elif types.is_numeric_dtype(self.__data[column]):
-                    if types.is_integer_dtype(self.__data[column]):
+                elif types.is_numeric_dtype(dtype):
+                    if types.is_integer_dtype(dtype):
                         current_type = "INTEGER"
-                    elif types.is_float_dtype(self.__data[column]):
+                    elif types.is_float_dtype(dtype):
                         current_type = "NUMBER"
 
                 new_object = {
@@ -1921,6 +2113,16 @@ class Provider(object):
         self.__error = False
         return self
 
+    def colgroupby(self, args):
+        for g in args:
+            if not isinstance(g, Attribute):
+                print('colgroupby function parameters must be an instance of a Attribute object')
+                self.__error = True
+                return self
+            self.__col_groups.append(g)
+        self.__error = False
+        return self
+
     def rows(self, args):
         for r in args:
             if isinstance(r, Row):
