@@ -929,190 +929,4 @@ class Constants(object):
         },
         "additionalProperties": True
     }
-    # region JAVASCRIPT FUNCTIONS
-    BS = chr(92)
-    SANITIZE_STRING = f""".replace(/{BS}{BS}'/g, "'").replace(/{BS}{BS}{BS}{BS}/g, "{BS}{BS}")"""
-    DEFINE_EXECUTE_PYTHON_JS_FUNC = """
-    window.executePython = function (python, kernelId, action = undefined, df = '', config = '') {
-        if (typeof Jupyter !== 'undefined') {
-            return new Promise((resolve, reject) => {
-                var callback = {
-                    iopub: {
-                        output: (data) => {
-                            if (data.content.text) {
-                                resolve(data.content.text.trim());
-                            } else if (data.content.ename && data.content.evalue) {
-                                if (data.content.evalue === "name 'cf' is not defined") {
-                                    data.content.evalue = "Name 'cf' is not defined. Please execute 'from chartfactor import *' in any cell above.";
-                                }
-                                resolve(`{"ename": "${data.content.ename}", "evalue": "${data.content.evalue}"}`);
-                            } else {
-                                resolve('');
-                            }
-                        }
-                    }
-                };
-    
-                Jupyter.notebook.kernel.execute(`${python}`, callback);
-            });
-        } else if (typeof JupyterLab !== 'undefined') {
-            return new Promise((resolve, reject) => {
-                if (JupyterLab[kernelId]) { // Running on JupyterLab environment
-                    JupyterLab[kernelId].notebook.kernel.execute(`${python}`, resolve);
-                } else if (JupyterLab.context.sessionContext && JupyterLab.context.sessionContext.session?.kernel) { // Running on Kaggle or any other environment that supports JupyterLab
-                    const future = (value) => {
-                        this._future = value;
-                        if (!value) {
-                            return;
-                        }
-    
-                        value.onIOPub = (msg) => {
-                            const msgType = msg.header.msg_type;
-    
-                            switch (msgType) {
-                                case 'execute_result':
-                                    if (resolve) {
-                                        const data = msg.content.data['text/plain'] || '';
-                                        resolve(data);
-                                    } else {
-                                        resolve('');
-                                    }
-                                    break;
-                                case 'display_data':
-                                    if (resolve) {
-                                        resolve('');
-                                    }
-                                    break;
-                                case 'update_display_data':
-                                    if (resolve) {
-                                        resolve('');
-                                    }
-                                    break;
-                                case 'stream':
-                                    if (resolve) {
-                                        resolve(msg.content.text.trim());
-                                    } else {
-                                        resolve('');
-                                    }
-                                    break;
-                                case 'error':
-                                    if (resolve && msg.content.ename && msg.content.evalue) {
-                                        if (msg.content.evalue === "name 'cf' is not defined") {
-                                            msg.content.evalue = "Name 'cf' is not defined. Please execute 'from chartfactor import *' in any cell above.";
-                                        }
-                                        resolve(`{"ename": "${msg.content.ename}", "evalue": "${msg.content.evalue}"}`);
-                                    } else {
-                                        resolve('');
-                                    }
-                                    break;
-                                default:
-                                    break;
-                            }
-                            return;
-                        };
-                    }
-    
-                    const code = `${python}`;
-                    future(JupyterLab.context.sessionContext.session?.kernel.requestExecute({code}));
-                } else {
-                    resolve(`{"ename": "Kernel", "evalue": "It appears that the Jupyter Lab Kernel of this notebook has changed since the last run. Please import chartactor and run the cell again."}`);
-                }
-            });
-        } else if (typeof google !== 'undefined' && google.colab) {
-            return new Promise((resolve, reject) => {
-                const processOutput = (response) => {
-                    if (response.ename && response.evalue) {
-                        if (response.evalue === "name 'cf' is not defined") {
-                            response.evalue = "Name 'cf' is not defined. Please execute 'from chartfactor import *' in any cell above.";
-                        }
-                        resolve(`{"ename": "${response.ename}", "evalue": "${response.evalue}"}`);
-                    } else if (response.data['text/plain']) {
-                        resolve(response.data['text/plain'].trim().slice(1, -1)""" + SANITIZE_STRING + """);
-                    } else {
-                        resolve('');
-                    }
-                }
-        
-                switch(action) {
-                    case "getDatasources":
-                        google.colab.kernel.invokeFunction("get_data_sources", [], {}).then(response => processOutput(response));
-                        break;
-                    case "getDatasource":
-                        google.colab.kernel.invokeFunction("get_data_source", [df], {}).then(response => processOutput(response));
-                        break;
-                    case "runCountQuery":
-                        google.colab.kernel.invokeFunction("run_count_query", [df, JSON.stringify(config)], {}).then(response => processOutput(response));
-                        break;
-                    case "runRawQuery":
-                        google.colab.kernel.invokeFunction("run_raw_query", [df, JSON.stringify(config)], {}).then(response => processOutput(response));
-                        break;
-                    case "visualize":
-                        google.colab.kernel.invokeFunction("visualize", [df, JSON.stringify(config)], {}).then(response => processOutput(response));
-                        break;
-                    default:
-                        break;
-                }
-            });
-        }
-    }
-    """
-    DEFINE_SEND_MESSAGE_JS_FUNC = """
-    window.sendMessage = function(r, fId, a, reqId) {          
-        try {
-            JSON.parse(r);
-        } catch (e) {
-            console.log(r);
-            r = '{"data": {"data": [], "visualization": ""}}'            
-        }    
-        
-        const iframe = document.getElementById(fId)
-        if(iframe) {
-            iframe.contentWindow.postMessage({response: r, action: a, reqId}, '*');
-        }
-    };
-    """
-    DEFINE_EVENT_LISTENER_FUNC = """
-    function cfMessageEventListener(event) {
-        let python_code = "";
-        
-        if (event.data.action === 'getDatasources') {
-            %s
-        } else if (event.data.action === 'getDatasource') {
-            %s      
-        } else if (event.data.action === 'visualize') {                    
-            %s
-        } else if (event.data.action === 'runCountQuery') {                    
-            %s
-        } else if (event.data.action === 'runRawQuery') {                    
-            %s
-        } else if (event.data.action === 'isLoadedInJupyter') {                    
-            const iframe = document.getElementById(event.data.iframe)
-            if(iframe) {
-                iframe.contentWindow.postMessage({response: 'OK', action: event.data.action, reqId: event.data.reqId}, '*');
-            }
-        } else if (event.data.action === 'resizeIframeHeight') {
-            document.getElementById(event.data.iframe).style.setProperty('height', `${event.data.iframeHeight + 100}px`);
-            if (typeof google !== 'undefined' && google.colab) {
-                const height = event.data.iframeHeight + 130; 
-                google.colab.output.setIframeHeight(height, true, {
-                    maxHeight: height,
-                });
-            }
-        }
-        
-        if (python_code !== "") {
-            let kernelId = event.data.iframe.split('_');
-            if (kernelId && kernelId.length === 2) kernelId = kernelId[1];
-            window.executePython(python_code, kernelId, event.data.action, event.data.df, event.data.conf).then(result => window.sendMessage(result, `${event.data.iframe}`, `${event.data.action}`, `${event.data.reqId}`));
-        }        
-    }
-    """
-    EXECUTE_PYTHON_COMMAND_JS = """
-    python_code = %s;    
-    """
-    GET_DATA_SOURCES_PY_COMMAND = "\"print(json.dumps([{'id': g, 'name': g, 'providerType': 'pandas-dataframe', 'type': 'DATASET' } for g in globals() if isinstance(eval(g), pd.core.frame.DataFrame)]))\""
-    GET_DATA_SOURCE_META_PY_COMMAND = "\"print(cf.provider(\"+ %s +\").get_data_source('\"+ %s +\"'))\""
-    VISUALIZE_PY_COMMAND = "\"print(cf.provider(\"+ %s +\", json.loads('''\"+ JSON.stringify(%s) +\"''')).visualize())\""
-    COUNT_QUERY_PY_COMMAND = "\"print(cf.provider(\"+ %s +\", json.loads('''\"+ JSON.stringify(%s) +\"''')).run_count_query())\""
-    RAW_QUERY_PY_COMMAND = "\"print(cf.provider(\"+ %s +\", json.loads('''\"+ JSON.stringify(%s) +\"''')).run_raw_query())\""
-    # endregion
+
