@@ -7,13 +7,14 @@ except ImportError:
 
 from ..assets import css
 from ..assets import js
-from ..assets.js import cft
+
 
 class StaticFilesManager(object):
     def __init__(self):
         pass
 
-    def js(self, file):
+    @staticmethod
+    def js(file):
         code = ''
         try:
             code = pkg_resources.read_text(js, f'{file}.js')
@@ -21,18 +22,11 @@ class StaticFilesManager(object):
             print(e)
         return code
 
-    def cft(self, file):
-        code = ''
-        try:
-            code = pkg_resources.read_text(cft, f'{file}.js')
-        except Exception as e:
-            print(e)
-        return code
-
-    def css(self):
+    @staticmethod
+    def css():
         styles = ''
         try:
             styles = pkg_resources.read_text(css, 'style.css')
         except Exception as e:
             print(e)
-        return styles
+        return styles
\ No newline at end of file
